/* SCCS  %W%---%E%--%U% */
//
//	nsl_stim_3d.c
//

# include "nsl_all_kernel_include.h"
# include "nsl_all_input_include.h"

nsl_stim_3d::nsl_stim_3d(nsl_input_3d* l) : nsl_stim_matrix(l),
	angle("angle",this,1), 	// stim original corner location
	z0("z0",this,1), 	// stim original corner location
	tz0("tz0",this,1), 	// stim current corner location
	zc("zc",this,1), 	// stim original center location
	tzc("tzc",this,1),	// stim current center location
	dz("dz",this,1), 	// stim size
	vz("vz",this,1) 	// stim speed	
{
	set_type("stim_3d");
	set_base_type("stim");

	angle = 0;
 	z0 = 0;
	tz0 = 0;
	zc = 0;
	tzc = 0;

	vz = 0.0;
}
nsl_stim_3d::~nsl_stim_3d()
{
}

int nsl_stim_3d::proc()
{
	return 1;
}

int  nsl_stim_3d::run()
{
 	float dt = 0;
	float ts = 0;
	int its = 0;

/*AW99	nsl_model* model = SYSTEM->get_model();
	if (model != NULL)
	{
		dt = model->getSimDelta();
		ts = model->getSimTime();
		its = model->getSimTimeStep();
	}
	else
		cmd_error("NULL MODEL");
*/
	if (NSL_SYSTEM != NULL)
	{
		dt = NSL_SYSTEM->getSimDelta();
		ts = NSL_SYSTEM->getSimTime();
		its = NSL_SYSTEM->getSimTimeStep();
	}
	else
		cmd_error("NULL NSL_SYSTEM");
		
	int dt0 = step;		// recalculation frequency
	if (dt0 == 0 && ts == 0.0)
	{
		txc = xc;
		tyc = yc;
		tx0 = x0;
		ty0 = y0;
	}
	else if (dt0 != 0 && its%step == 0)
	{
		txc = xc + ts*vx;
		tyc = yc + ts*vy;
		tx0 = x0 + ts*vx;
		ty0 = y0 + ts*vy;
	}

	if (mode == 0)
		proc(); // virtual
	else if (time_list != NULL)
	{
	   time_list->wind_up();
	   nsl_time_interval* time_obj;
	   while (time_obj = (nsl_time_interval*)time_list->next())
	   {
	    	if (time_obj->get_able_fg() == 0)
			continue;

		float t0 = time_obj->get_t0();
		float t1 = time_obj->get_t1();

		if ((ts >= t0 && ts <= t1))
		{
			proc(); // virtual
		}
	   }
	}
	else
	   cmd_out("NULL 'time_list'");

	return 1;
}
int nsl_stim_3d::print(ostream& out) const
{
 	return print_stim_3d(out);
}
int nsl_stim_3d::print_stim_3d(ostream& out) const
{
	nsl_stim_matrix::print(out);

	out << "set stim z0 " << z0.get_value() << "\n";
	out << "set stim zc " << zc.get_value() << "\n";
 	out << "set stim vz " << vz.get_value() << "\n";
	out.flush();

	return 1;
}
int nsl_stim_3d::print_status(ostream& out) const
{
 	return print_stim_3d_status(out);
}
int nsl_stim_3d::print_stim_3d_status(ostream& out) const
{
	nsl_stim_matrix::print_status(out);

 	out << "// stim tz0: " << tz0.get_value() << "\n";
	out << "// stim tzc: " << tzc.get_value() << "\n";
	out.flush();

	return 1;
}
 

