/* SCCS  @(#)nsl_data_mult_library.C	1.1---95/08/13--20:48:13 */
//
//	nsl_data_mult_library.C
//

# include "nsl_all_kernel_include.h"

// nsl_data multiplication

nsl_data operator^(const nsl_data& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_data  operator^\n";
	cout << "// (data) " << a.get_name() << " ^ (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	nsl_data c;
	c = a.get_data() * b.get_data();
	return c;
}
nsl_data operator^(const nsl_data& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_data  operator^\n";
	cout << "// (data) " << a.get_name() << " ^ (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	nsl_data c;
	c = a.get_data() * b;
	return c;
}
nsl_data operator^(const num_type a, const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_data  operator^\n";
	cout << "// (num_type) " << a << " ^ (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	nsl_data c;
	c = a * b.get_data();
	return c;
}

// mult_equal

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() = a.get_data() * b;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	out = 0;
	for (int i = 0; i < amax; i++)
		out = out + a[i] * b;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	out = 0;
	for (int i = 0; i < amax; i++)
		out = out + a[i] * b[i];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	out = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out = out + a[i][j] * b;
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	out = 0;
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out = out + a[i][j] * b[i][j];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() = a.get_data() * b.get_data();
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	out = 0;
	for (int i = 0; i < bmax; i++)
		out = out + a * b[i];
}

void NSLmult_equal(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	out = 0;
	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		out = out + a * b[i][j];
}

// mult_plus

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() += a.get_data() * b;
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	for (int i = 0; i < amax; i++)
		out = out + a[i] * b.get_data();
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	for (int i = 0; i < amax; i++)
		out = out + a[i] * b[i];
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() += a.get_data() * b.get_data();
}

void NSLmult_plus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	for (int i = 0; i < bmax; i++)
		out = out + a.get_data() * b[i];
}

// mult_minus

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,const num_type& b)
{
	out.elem() -= a.get_data() * b;
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int amax = a.get_imax();

	for (int i = 0; i < amax; i++)
		out = out - a[i] * b.get_data();
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int amax = a.get_imax();

	for (int i = 0; i < amax; i++)
		out = out - a[i] * b[i];
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	out.elem() -= a.get_data() * b.get_data();
}

void NSLmult_minus(nsl_num_data& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int bmax = b.get_imax();

	for (int i = 0; i < bmax; i++)
		out = out - a.get_data() * b[i];
}
