/* SCCS  @(#)nsl_matrix_mult_library.C	1.1---95/08/13--20:48:16 */
//
//	nsl_matrix_mult_library.C
//

# include "nsl_all_kernel_include.h"

// matrix pointwise multiplication

nsl_matrix operator^(const nsl_matrix& a,const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator^\n";
        cout << "// (matrix) " << a.get_name() << " ^ (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("*** nsl_matrix::operator^ ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return a;
	}

	nsl_matrix c(aimax,ajmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b[i][j];

	return c;
}
nsl_matrix operator^(const nsl_matrix& a,const num_type b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator^\n";
        cout << "// (matrix) " << a.get_name() << " ^ (num_type) ";
        cout << b << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b;

	return c;
}
nsl_matrix operator^(const num_type a, const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator^\n";
        cout << "// (num_type) " << a << " ^ (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		c[i][j] = a * b[i][j];

	return c;
}
nsl_matrix operator^(const nsl_matrix& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator^\n";
        cout << "// (matrix) " << a.get_name() << " ^ (data) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);
	
	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b.get_data();

	return c;
}
nsl_matrix operator^(const nsl_data& a, const nsl_matrix& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_matrix operator^\n";
        cout << "// (data) " << a.get_name() << " ^ (matrix) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		c[i][j] = a.get_data() * b[i][j];

	return c;
}

// matrix pointwise multiplication

nsl_matrix NSLmult(const nsl_matrix& a,const nsl_matrix& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != bimax || ajmax != bjmax)
	{
		cmd_error("*** nsl_matrix::NSLmult ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent matrix sizes");
		return a;
	}

	nsl_matrix c(aimax,ajmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b[i][j];

	return c;
}
nsl_matrix NSLmult(const nsl_matrix& a,const num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b;

	return c;
}
nsl_matrix NSLmult(const num_type a, const nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		c[i][j] = a * b[i][j];

	return c;
}
nsl_matrix NSLmult(const nsl_matrix& a,const nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	nsl_matrix c(aimax,ajmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		c[i][j] = a[i][j] * b.get_data();

	return c;
}
nsl_matrix NSLmult(const nsl_data& a,const  nsl_matrix& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	nsl_matrix c(bimax,bjmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
		c[i][j] = a.get_data() * b[i][j];

	return c;
}

// mult_equal

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a[i][j] * b;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a[i][j] * b.get_data();
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& a,const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a[i][j] * b[i][j];
}

// matrix conv
// fix for no allocation/deallocation of d

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_matrix& b,const nsl_num_vector& a)
{
	int saimax = a.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	int vec_type;
	int simax;
	int m;
	num_type val;

	val = 0;
	out = 0; // needed

	if ((vec_type = a.get_vec_type()) == 0)
	{
		simax = saimax + sbjmax - 1;
		nsl_matrix d(sbimax,simax);

		d.put_sector(b,0,saimax/2); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;	
			for (m = 0; m < saimax; m++)
			    val = val + a[m] * d[i][j+m];
			out[i][j] = val;
		   }
	}
	else
	{
		simax = saimax + sbimax - 1;
		nsl_matrix d(simax,sbjmax);

		d.put_sector(b,saimax/2,0); 

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
		   {	
			val = 0;
			for (m = 0; m < saimax; m++)
			    val = val + a[m] * d[i+m][j];
			out[i][j] = val;
		   }
	}
}

// vec to mat

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_vector& b,const nsl_num_data& a)
{
	int sbimax = b.get_imax();
	int simax = out.get_imax();
	int sjmax = out.get_jmax();

	if (b.get_vec_type() == 0) // ROW
	{
		if (sbimax != sjmax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < simax; i++)
		   for (int j = 0; j < sjmax; j++)
			out[i][j] = a.get_data()*b[j];
	}
	else // COL
	{
		if (sbimax != simax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < simax; i++)
		    for (int j = 0; j < sjmax; j++)
			out[i][j] = a.get_data()*b[i];
	}
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a.get_data() * b;
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a.get_data() * b.get_data();
}

void NSLmult_equal(nsl_num_matrix& out,const nsl_num_data& a,const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = a.get_data() * b[i][j];
}

// mult plus

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a[i][j] * b;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a[i][j] * b.get_data();
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a[i][j] * b[i][j];
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a.get_data() * b;
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a.get_data() * b.get_data();
}

void NSLmult_plus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_plus: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] + a.get_data() * b[i][j];
}

// mult minus

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a[i][j] * b;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();

	if (aimax != outimax || ajmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a[i][j] * b.get_data();
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_matrix& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (aimax != outimax || ajmax != outjmax || 
		aimax != bimax || ajmax != bjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a[i][j] * b[i][j];
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a.get_data() * b;
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a.get_data() * b.get_data();
}

void NSLmult_minus(nsl_num_matrix& out,const nsl_num_data& a,
	const nsl_num_matrix& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();

	if (bimax != outimax || bjmax != outjmax)
	{
		cmd_error("NSLmult_minus: Inconsistent matrix sizes");
		out.print_status(cerr);
		b.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
	   for (int j = 0; j < outjmax; j++)
		out[i][j] = out[i][j] - a.get_data() * b[i][j];
}

