/* SCCS  @(#)nsl_num_3_add_library.C	1.1+++95/08/13++20:38:16 */
//
//	nsl_num_3_add_library.C
//

# include "nsl_all_kernel_include.h"

// num_3 addtruction

nsl_num_3 operator+(const nsl_num_3& a)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (num_3) " << " = + (num_3) ";
        cout << a.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
		for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = +a[i][j][k];

	return c;
}
nsl_num_3 operator+(const nsl_num_3& a,const nsl_num_3& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (num_3) " << a.get_name() << " + (num_3) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	if (aimax != bimax || ajmax != bjmax || akmax != bkmax)
	{
		cmd_error("*** nsl_num_3::operator+ ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent num_3 sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b[i][j][k];

	return c;
}
nsl_num_3 operator+(const nsl_num_3& a,const num_type b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (num_3) " << a.get_name() << " + (num_type) ";
        cout << b << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b;

	return c;
}
nsl_num_3 operator+(const num_type a, const nsl_num_3& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (num_type) " << a << " + (num_3) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(bimax,bjmax,bkmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
					c[i][j][k] = a + b[i][j][k];

	return c;
}
nsl_num_3 operator+(const nsl_num_3& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (num_3) " << a.get_name() << " + (data) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b.get_data();

	return c;
}
nsl_num_3 operator+(const nsl_data& a, const nsl_num_3& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_3 operator+\n";
        cout << "// (data) " << a.get_name() << " + (num_3) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(bimax,bjmax,bkmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
					c[i][j][k] = a.get_data() + b[i][j][k];

	return c;
}

// num_3 addtraction

nsl_num_3 NSLadd(const nsl_num_3& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = +a[i][j][k];

	return c;
}
nsl_num_3 NSLadd(const nsl_num_3& a,const nsl_num_3& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	if (aimax != bimax || ajmax != bjmax || akmax != bkmax)
	{
		cmd_error("*** nsl_num_3::NSLadd ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent num_3 sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b[i][j][k];

	return c;
}
nsl_num_3 NSLadd(const nsl_num_3& a,const num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b;

	return c;
}
nsl_num_3 NSLadd(const num_type a, const nsl_num_3& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(bimax,bjmax,bkmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
					c[i][j][k] = a + b[i][j][k];

	return c;
}
nsl_num_3 NSLadd(const nsl_num_3& a,const nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();

	nsl_num_3 c(aimax,ajmax,akmax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
					c[i][j][k] = a[i][j][k] + b.get_data();

	return c;
}
nsl_num_3 NSLadd(const nsl_data& a,const  nsl_num_3& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();

	nsl_num_3 c(bimax,bjmax,bkmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
					c[i][j][k] = a.get_data() + b[i][j][k];

	return c;
}

