/* SCCS  @(#)nsl_num_4_add_library.C	1.1+++95/08/13++20:48:16 */
//
//	nsl_num_4_add_library.C
//

# include "nsl_all_kernel_include.h"

// num_4 addtruction

nsl_num_4 operator+(const nsl_num_4& a)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (num_4) " << " = + (num_4) ";
        cout << a.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
		for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = +a[i][j][k][l];

	return c;
}
nsl_num_4 operator+(const nsl_num_4& a,const nsl_num_4& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (num_4) " << a.get_name() << " + (num_4) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	if (aimax != bimax || ajmax != bjmax || akmax != bkmax || almax != blmax)
	{
		cmd_error("*** nsl_num_4::operator+ ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent num_4 sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b[i][j][k][l];

	return c;
}
nsl_num_4 operator+(const nsl_num_4& a,const num_type b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (num_4) " << a.get_name() << " + (num_type) ";
        cout << b << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b;

	return c;
}
nsl_num_4 operator+(const num_type a, const nsl_num_4& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (num_type) " << a << " + (num_4) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(bimax,bjmax,bkmax,blmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
				for (int l = 0; l < blmax; l++)
					c[i][j][k][l] = a + b[i][j][k][l];

	return c;
}
nsl_num_4 operator+(const nsl_num_4& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (num_4) " << a.get_name() << " + (data) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b.get_data();

	return c;
}
nsl_num_4 operator+(const nsl_data& a, const nsl_num_4& b)
{
#ifdef NSL_DEBUG
        cout << "// nsl_num_4 operator+\n";
        cout << "// (data) " << a.get_name() << " + (num_4) ";
        cout << b.get_name() << ";\n";
        cout.flush();
#endif

	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(bimax,bjmax,bkmax,blmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
				for (int l = 0; l < blmax; l++)
					c[i][j][k][l] = a.get_data() + b[i][j][k][l];

	return c;
}

// num_4 addtraction

nsl_num_4 NSLadd(const nsl_num_4& a)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = +a[i][j][k][l];

	return c;
}
nsl_num_4 NSLadd(const nsl_num_4& a,const nsl_num_4& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	if (aimax != bimax || ajmax != bjmax || akmax != bkmax || almax != blmax)
	{
		cmd_error("*** nsl_num_4::NSLadd ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent num_4 sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b[i][j][k][l];

	return c;
}
nsl_num_4 NSLadd(const nsl_num_4& a,const num_type b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b;

	return c;
}
nsl_num_4 NSLadd(const num_type a, const nsl_num_4& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(bimax,bjmax,bkmax,blmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
				for (int l = 0; l < blmax; l++)
					c[i][j][k][l] = a + b[i][j][k][l];

	return c;
}
nsl_num_4 NSLadd(const nsl_num_4& a,const nsl_data& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	nsl_num_4 c(aimax,ajmax,akmax,almax);

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					c[i][j][k][l] = a[i][j][k][l] + b.get_data();

	return c;
}
nsl_num_4 NSLadd(const nsl_data& a,const  nsl_num_4& b)
{
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	nsl_num_4 c(bimax,bjmax,bkmax,blmax);

	for (int i = 0; i < bimax; i++)
	   for (int j = 0; j < bjmax; j++)
			for (int k = 0; k < bkmax; k++)
				for (int l = 0; l < blmax; l++)
					c[i][j][k][l] = a.get_data() + b[i][j][k][l];

	return c;
}

void NSLadd_plus(const nsl_num_4& a,const nsl_num_4& b)
{
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();
	int bimax = b.get_imax();
	int bjmax = b.get_jmax();
	int bkmax = b.get_kmax();
	int blmax = b.get_lmax();

	if (aimax != bimax || ajmax != bjmax || akmax != bkmax || almax != blmax)
	{
		cmd_error("*** nsl_num_4::NSLadd ***");
		cmd_error("Exit with the following status");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent num_4 sizes");
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
			for (int k = 0; k < akmax; k++)
				for (int l = 0; l < almax; l++)
					a[i][j][k][l] = a[i][j][k][l] + b[i][j][k][l];
}
