//
//	nsl_num_4_mult_library.C
//

# include "nsl_all_kernel_include.h"

void NSLmult_equal(nsl_num_4& out,const nsl_num_4& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int outkmax = out.get_kmax();
	int outlmax = out.get_lmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	if (aimax != outimax || ajmax != outjmax || akmax != outkmax || almax != outlmax)
	{
		cmd_error("NSLmult_equal: Inconsistent num_4 sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
		for (int j = 0; j < outjmax; j++)
			for (int k = 0; k < outkmax; k++)
				for (int l = 0; l < outlmax; l++)				
					out[i][j][k][l] = a[i][j][k][l] * b;
}

void NSLmult_equal(nsl_num_4& out,const num_type& b,
	const nsl_num_4& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int outkmax = out.get_kmax();
	int outlmax = out.get_lmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	if (aimax != outimax || ajmax != outjmax || akmax != outkmax || almax != outlmax)
	{
		cmd_error("NSLmult_equal: Inconsistent num_4 sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
		for (int j = 0; j < outjmax; j++)
			for (int k = 0; k < outkmax; k++)
				for (int l = 0; l < outlmax; l++)				
					out[i][j][k][l] = a[i][j][k][l] * b;
}

void NSLmult_plus(nsl_num_4& out,const nsl_num_4& a,
	const num_type& b)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int outkmax = out.get_kmax();
	int outlmax = out.get_lmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	if (aimax != outimax || ajmax != outjmax || akmax != outkmax || almax != outlmax)
	{
		cmd_error("NSLmult_plus: Inconsistent num_4 sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
		for (int j = 0; j < outjmax; j++)
			for (int k = 0; k < outkmax; k++)
				for (int l = 0; l < outlmax; l++)				
					out[i][j][k][l] = out[i][j][k][l] + a[i][j][k][l] * b;
}

void NSLmult_plus(nsl_num_4& out,const num_type& b,
	const nsl_num_4& a)
{
	int outimax = out.get_imax();
	int outjmax = out.get_jmax();
	int outkmax = out.get_kmax();
	int outlmax = out.get_lmax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int akmax = a.get_kmax();
	int almax = a.get_lmax();

	if (aimax != outimax || ajmax != outjmax || akmax != outkmax || almax != outlmax)
	{
		cmd_error("NSLmult_plus: Inconsistent num_4 sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outimax; i++)
		for (int j = 0; j < outjmax; j++)
			for (int k = 0; k < outkmax; k++)
				for (int l = 0; l < outlmax; l++)				
					out[i][j][k][l] = out[i][j][k][l] + a[i][j][k][l] * b;
}

