/* SCCS  @(#)nsl_vector_add_library.C	1.1---95/08/13--20:48:18 */
//
//	nsl_vector_add_library.C
//

# include "nsl_all_kernel_include.h"

nsl_vector operator+(const nsl_vector& a)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (vector) " << " = (vector) ";
	cout << a.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i];

	return c;
}
nsl_vector operator+(const nsl_vector& a,const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (vector) " << a.get_name() << " + (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator+ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b[i];

	return c;
}
nsl_vector operator+(const nsl_vector& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (vector) " << a.get_name() << " + (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b;

	return c;
}
nsl_vector operator+(const num_type a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (num_type) " << a << " + (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a + b[i];

	return c;
}
nsl_vector operator+(const nsl_vector& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (vector) " << a.get_name() << " + (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b.get_data();

	return c;
}
nsl_vector operator+(const nsl_data& a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator+\n";
	cout << "// (data) " << a.get_name() << " + (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a.get_data() + b[i];

	return c;
}

// vector addition

nsl_vector NSLadd(const nsl_vector& a)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i];

	return c;
}
nsl_vector NSLadd(const nsl_vector& a,const nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::NSLadd ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b[i];

	return c;
}
nsl_vector NSLadd(const nsl_vector& a,const num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b;

	return c;
}
nsl_vector NSLadd(const num_type a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a + b[i];

	return c;
}
nsl_vector NSLadd(const nsl_vector& a,const nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] + b.get_data();

	return c;
}
nsl_vector NSLadd(const nsl_data& a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a.get_data() + b[i];

	return c;
}

// add equal

void NSLadd_equal(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a[i];
}
void NSLadd_equal(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a.get_data();
}

// mat_to_vec

void NSLadd_equal(nsl_num_vector& out,const nsl_num_matrix& a)
{
	int vec_type = out.get_vec_type();

	int simax = a.get_imax();
	int sjmax = a.get_jmax();

	int s = out.get_imax();

	out = 0;
	
	if (vec_type == 0) // row vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    out[j] = out[j] + a[i][j];
	}
	if (vec_type == 1) // col vector
	{
	    for (int i = 0; i < simax; i++)
	    	for (int j = 0; j < sjmax; j++)
		    out[i] = out[i] + a[i][j];
	}
}

// add plus

void NSLadd_plus(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a[i];
}
void NSLadd_plus(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a.get_data();
}

// add minus

void NSLadd_minus(nsl_num_vector& out,const nsl_num_vector& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a[i];
}
void NSLadd_minus(nsl_num_vector& out,const nsl_num_data& a)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a.get_data();
}

