/* SCCS  @(#)nsl_vector_mult_library.C	1.1---95/08/13--20:48:22 */
//
//	nsl_vector_mult_library.C
//

# include "nsl_all_kernel_include.h"

// vector pointwise multiplication

nsl_vector operator^(const nsl_vector& a,const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator^\n";
	cout << "// (vector) " << a.get_name() << " ^ (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::operator^ ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b[i];

	return c;
}
nsl_vector operator^(const nsl_vector& a,const num_type b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator^\n";
	cout << "// (vector) " << a.get_name() << " ^ (num_type) ";
	cout << b << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b;

	return c;
}
nsl_vector operator^(const num_type a, const nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator^\n";
	cout << "// (num_type) " << a << " ^ (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a * b[i];

	return c;
}
nsl_vector operator^(const nsl_vector& a,const nsl_data& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator^\n";
	cout << "// (vector) " << a.get_name() << " ^ (data) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b.get_data();

	return c;
}
nsl_vector operator^(const nsl_data& a,const  nsl_vector& b)
{
#ifdef NSL_DEBUG
	cout << "// nsl_vector  operator^\n";
	cout << "// (data) " << a.get_name() << " ^ (vector) ";
	cout << b.get_name() << ";\n";
	cout.flush();
#endif

	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a.get_data() * b[i];

	return c;
}

// vector pointwise multiplication

nsl_vector NSLmult(const nsl_vector& a,const nsl_vector& b)
{
	int aimax = a.get_imax();
	int bimax = b.get_imax();

	nsl_vector c(aimax);

	if (aimax != bimax)
	{
		cmd_error("*** nsl_vector::NSLmult ***");
		cmd_error("Exit with status:");
		a.print_status(cerr);
		b.print_status(cerr);
		cmd_error("Inconsistent vector sizes");
		return c;
	}

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b[i];

	return c;
}
nsl_vector NSLmult(const nsl_vector& a,const num_type b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b;

	return c;
}
nsl_vector NSLmult(const num_type a, const nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a * b[i];

	return c;
}
nsl_vector NSLmult(const nsl_vector& a,const nsl_data& b)
{
	int aimax = a.get_imax();

	nsl_vector c(aimax);

	for (int i = 0; i < aimax; i++)
		c[i] = a[i] * b.get_data();

	return c;
}
nsl_vector NSLmult(const nsl_data& a,const  nsl_vector& b)
{
	int bimax = b.get_imax();

	nsl_vector c(bimax);

	for (int i = 0; i < bimax; i++)
		c[i] = a.get_data() * b[i];

	return c;
}

// mult equal

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a[i] * b;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a[i] * b.get_data();
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();
	int amax = a.get_imax();
	int bmax = b.get_imax();

	if (amax != bmax || bmax != outmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < outmax; i++)
		out[i] = a[i] * b[i];
}

// mat to vec

void NSLmult_equal(nsl_num_vector& out,const nsl_num_matrix& b,
	const nsl_num_data& a)
{
	int simax = out.get_imax();
	int sbimax = b.get_imax();
	int sbjmax = b.get_jmax();

	out = 0;

	if (out.get_vec_type() == 0) // ROW
	{
		if (simax != sbjmax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < sbimax; i++)
		   for (int j = 0; j < sbjmax; j++)
			out[j] = out[j] + a.get_data()*b[i][j];
	}
	else // COL
	{
		if (simax != sbimax)
		{
			cmd_error("NSLmult_equal: Inconsistent matrix sizes");
			out.print_status(cerr);
			b.print_status(cerr);
		}

		for (int i = 0; i < sbimax; i++)
		    for (int j = 0; j < sbjmax; j++)
			out[i] = out[i] + a.get_data()*b[i][j];
	}
}
void NSLmult_equal(nsl_num_vector& out,const nsl_num_matrix& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();
	int aimax = a.get_imax();
	int ajmax = a.get_jmax();
	int bmax = b.get_imax();

	if (aimax != bmax || bmax != outmax)
	{
		cmd_error("NSLmult_equal: Inconsistent matrix sizes");
		out.print_status(cerr);
		a.print_status(cerr);
	}

	for (int i = 0; i < aimax; i++)
	   for (int j = 0; j < ajmax; j++)
		out[i] = a[i][j] * b[i];
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a.get_data() * b.get_data();
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a.get_data() * b;
}

void NSLmult_equal(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = a.get_data() * b[i];
}

// mult plus

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a[i] * b;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a[i] * b.get_data();
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a[i] * b[i];
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a.get_data() * b;
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a.get_data() * b.get_data();
}

void NSLmult_plus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] + a.get_data() * b[i];
}

// mult minus

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a[i] * b;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a[i] * b.get_data();
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_vector& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a[i] * b[i];
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const num_type& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a.get_data() * b;
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_data& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a.get_data() * b.get_data();
}

void NSLmult_minus(nsl_num_vector& out,const nsl_num_data& a,
	const nsl_num_vector& b)
{
	int outmax = out.get_imax();

	for (int i = 0; i < outmax; i++)
		out[i] = out[i] - a.get_data() * b[i];
}

